/* eslint-disable react-hooks/exhaustive-deps */
/* eslint-disable react-native/no-inline-styles */
import {FlatList, PanResponder, StyleSheet, View} from 'react-native';
import React, {createRef, useEffect, useRef, useState} from 'react';

import Fade from '../../../transitions/Fade';

import {Button} from 'react-native-paper';

const offsetToPositionCenter = (buttonWidth, containerWidth) => {
  return Math.floor(containerWidth / 2) - Math.floor(buttonWidth / 2);
};

const opacityFactor = (activeIndex, currentIndex) => {
  if (activeIndex - currentIndex === 0) {
    return 1;
  } else if (Math.abs(activeIndex - currentIndex) < 4) {
    return 1 - 0.3 * Math.abs(activeIndex - currentIndex);
  } else {
    return 0.2;
  }
};

const opacityFactorForFreeflow = (activeIndex, currentIndex) => {
  if (activeIndex === currentIndex) {
    return 1;
  }
  return 0.5;
};

const StepList = props => {
  // used to pull the step state from redux store
  const {steps, setActiveStep, activeStepIndex} = props;
  const freeflow = Boolean(props.freeflow);

  // used to store container width
  // used for calculating initial and final position offsets in step items
  const [containerWidth, setContainerWidth] = useState(0);

  // contains a ref of the actual stepList ListView to trigger ref functions on it
  const ref = useRef(null);

  // contains animation handle refs for each animated Component wrapper on list item
  // to allow for transitions to be triggered declaratively
  const [fadeRefs, setFadeRefs] = useState([]);

  // contains width value for each button, (since due to different word length)
  // center position via offset requires unique calculation
  // for each button
  const [buttonWidths, setButtonWidths] = useState([]);

  // contains the actual gesture value ("right" | "left")
  // with a timestamp applied to allow for re-setting of same
  // type of gesture
  const [gesture, setGesture] = useState('');

  // swipe sensitivity used, this is used to trigger swipe
  // only after the finger has moved across a certain threshold of distance
  const sensitivity = 15;

  const panResponder = useRef(
    PanResponder.create({
      onMoveShouldSetPanResponder: () => !freeflow,
      onMoveShouldSetPanResponderCapture: () => !freeflow,
      onPanResponderRelease: (_, gestureState) => {
        if (!freeflow) {
          // used to calculate if the user has swiped left/right
          if (gestureState.dx > sensitivity) {
            setGesture('right%SEPARATE%' + Date.now().toString());
          } else if (gestureState.dx < -sensitivity) {
            setGesture('left%SEPARATE%' + Date.now().toString());
          }
        }
      },
      onPanResponderTerminationRequest: () => !freeflow,
    }),
  ).current;

  // used to run a effect, based on the gesture user has selected
  useEffect(() => {
    const g = gesture.split('%SEPARATE%')[0];
    // edge case to allow left swipe to move to next step only if its not the last step in state list
    // edge case to allow right swipe to move to first step only if its not the first step in state list
    if (g === 'left') {
      setActiveStep(activeStepIndex + 1);
    } else if (g === 'right') {
      setActiveStep(activeStepIndex - 1);
    }
  }, [gesture]);

  // used to set the ref handle placeholders and buttonWidth placeholders
  // for animation on each item to be triggered on component mount
  useEffect(() => {
    setFadeRefs([
      ...Array(steps.length)
        .fill(null)
        .map((_, __) => createRef()),
    ]);
    setButtonWidths([
      ...Array(steps.length)
        .fill(null)
        .map((_, __) => 0),
    ]);
  }, [steps.length]);

  // effect to display on the UI which step is active and in center
  useEffect(() => {
    if (
      containerWidth > 0 &&
      steps.length > 0 &&
      buttonWidths.length === steps.length
    ) {
      // makes the listView declaratively scroll onto the given index
      // with a calculated offset

      setImmediate(() => {
        ref.current?.scrollToIndex({
          index: activeStepIndex,
          viewOffset: offsetToPositionCenter(
            buttonWidths[activeStepIndex],
            containerWidth,
          ),
        });
      });

      // triggers animation on each step declaratively based on its index
      fadeRefs.forEach((fadeRef, i) => {
        fadeRef.current?.triggerAnimation(
          freeflow
            ? opacityFactorForFreeflow(activeStepIndex, i)
            : opacityFactor(activeStepIndex, i),
        );
      });
    }
  }, [buttonWidths, containerWidth, activeStepIndex]);

  return (
    <View
      // used to record width of container in which this step list resides
      style={styles.root}
      onLayout={e => setContainerWidth(e.nativeEvent.layout.width)}>
      <FlatList
        {...panResponder.panHandlers}
        contentContainerStyle={styles.listStyle}
        ref={ref}
        horizontal
        scrollEnabled={freeflow}
        onScrollToIndexFailed={() => console.log('scroll error')}
        showsVerticalScrollIndicator={false}
        showsHorizontalScrollIndicator={false}
        data={steps}
        renderItem={data => {
          return (
            <Fade
              ref={fadeRefs[data.index]}
              config={{
                duration: 300,
                startOpacity: 0,
                endOpacity: freeflow
                  ? opacityFactorForFreeflow(activeStepIndex, data.index)
                  : opacityFactor(activeStepIndex, data.index),
              }}>
              <Button
                onLayout={e => {
                  // records width of each button
                  const widthState = [...buttonWidths];
                  widthState[data.index] = e.nativeEvent.layout.width;
                  setButtonWidths([...widthState]);
                }}
                style={{
                  // sets a initial offset from left or right based on position of element
                  marginRight:
                    data.index === steps.length - 1
                      ? offsetToPositionCenter(
                          buttonWidths[data.index]
                            ? buttonWidths[data.index]
                            : 0,
                          containerWidth,
                        )
                      : 10,
                  marginLeft:
                    data.index === 0
                      ? offsetToPositionCenter(
                          buttonWidths[data.index]
                            ? buttonWidths[data.index]
                            : 0,
                          containerWidth,
                        )
                      : 0,
                }}
                mode="text"
                textColor={data.item.isComplete ? 'green' : 'white'}>
                {data.item.title}
              </Button>
            </Fade>
          );
        }}
      />
    </View>
  );
};

export default StepList;

const styles = StyleSheet.create({
  root: {
    width: '100%',
    backgroundColor: 'transparent',
    position: 'absolute',
    bottom: 0,
    left: 0,
    zIndex: 100,
  },
  listStyle: {
    paddingBottom: 10,
    paddingTop: 10,
  },
});
