import {StyleSheet, Text, View} from 'react-native';
import React, {
  RefObject,
  createRef,
  forwardRef,
  useEffect,
  useMemo,
  useState,
} from 'react';
import {
  Form,
  FormField as FormFieldType,
  MediaField,
  RadioGroupField,
} from '@s20.ai/copilot-camera-sdk-with-frameprocessor/src/types/Widgets';
import BottomSheet, {BottomSheetFlatList} from '@gorhom/bottom-sheet';
import {FormField, FormFieldRef, RadioGroup} from './form/FormFields';
import {useCopilotCameraSlot} from '@s20.ai/copilot-camera-sdk-with-frameprocessor';
import {Slot} from '@s20.ai/copilot-camera-sdk-with-frameprocessor/src/types/AOP';
import {Button, Surface} from 'react-native-paper';
import Media, {MediaRef} from './Media';
import {WidgetDrawerRef} from '../../components/widgets';
import {StyleProp} from 'react-native';
import {ViewStyle} from 'react-native';

type Props = {
  onSubmit?: (widgets: Form[]) => void;
  onChange?: (index: number) => void;
  containerStyle?: StyleProp<ViewStyle>;
};

const BottomDrawer = forwardRef<WidgetDrawerRef, Props>(
  (props, bottomSheetRef) => {
    const [formFieldRefs, setFormFieldRefs] = useState<
      RefObject<FormFieldRef | MediaRef>[][]
    >([]);

    const snapPoints = useMemo(() => [0.01, '90%'], []);

    const slotHook = useCopilotCameraSlot();

    const FormData =
      slotHook.activeSlot && slotHook.activeSlot.widgets
        ? slotHook.activeSlot.widgets
        : undefined;

    useEffect(() => {
      setFormFieldRefs(_ => []);
      if (slotHook.activeSlot) {
        if (slotHook.activeSlot.widgets !== undefined) {
          const formLength = ((slotHook.activeSlot as Slot).widgets as Form[])
            .length;
          let arr = [];
          for (let i = 0; i < formLength; i++) {
            const formSectionLength = (
              (slotHook.activeSlot as Slot).widgets as Form[]
            )[i].fields.length;
            let arr_2 = [];
            for (let k = 0; k < formSectionLength; k++) {
              arr_2.push(createRef());
            }
            arr.push(arr_2);
          }
          setFormFieldRefs(arr as any);
        }
      }
    }, [slotHook.activeSlot]);

    const renderField = (
      field: FormFieldType | RadioGroupField | MediaField,
      ref: RefObject<FormFieldRef | MediaRef>,
    ) => {
      switch (field.type) {
        case 'number':
        case 'text':
          return (
            <FormField
              ref={ref as RefObject<FormFieldRef>}
              label={field.label}
              initialValue={field.value}
            />
          );
        case 'media':
          return (
            <>
              {slotHook.activeSlot &&
                slotHook.activeSlot.action &&
                slotHook.activeSlot.action.storagePath && (
                  <Media
                    ref={ref as RefObject<MediaRef>}
                    captureType={slotHook.activeSlot.action.captureType}
                    sourceUrl={
                      (slotHook.activeSlot.isUploaded
                        ? slotHook.activeSlot.uploadURL
                        : slotHook.activeSlot.action.storagePath) as string
                    }
                  />
                )}
            </>
          );
        default:
          return (
            <RadioGroup
              options={field.options}
              ref={ref as RefObject<FormFieldRef>}
              label={field.label}
              initialValue={field.value}
            />
          );
      }
    };

    return (
      <>
        {FormData !== undefined && formFieldRefs.length === FormData.length && (
          <BottomSheet
            ref={bottomSheetRef as RefObject<BottomSheet>}
            containerStyle={props.containerStyle}
            index={-1}
            snapPoints={snapPoints}
            onChange={props.onChange}>
            <View style={styles.viewContainer}>
              <BottomSheetFlatList
                data={FormData}
                renderItem={({item, index}) => {
                  return (
                    <Surface style={styles.contentContainer} key={item.title}>
                      <Text style={styles.text}>{item.title}</Text>
                      {item.fields.map((field, i) => (
                        <React.Fragment key={field.id}>
                          {renderField(field, formFieldRefs[index][i])}
                        </React.Fragment>
                      ))}

                      {index === FormData.length - 1 && (
                        <Button
                          onPress={() => {
                            const updatedForm = FormData.map(
                              (form, formIndex) => ({
                                ...form,
                                fields: form.fields.map((field, fieldIndex) => {
                                  return {
                                    ...field,
                                    value: (
                                      formFieldRefs[formIndex][fieldIndex]
                                        .current as any
                                    ).get(),
                                  };
                                }),
                              }),
                            ) as Form[];

                            if (props.onSubmit) {
                              props.onSubmit(updatedForm);
                            }
                          }}>
                          Submit
                        </Button>
                      )}
                    </Surface>
                  );
                }}
              />
            </View>
          </BottomSheet>
        )}
      </>
    );
  },
);

export default BottomDrawer;

const styles = StyleSheet.create({
  contentContainer: {
    flex: 1,
    paddingHorizontal: 20,
    marginBottom: 20,
    padding: 10,
  },
  viewContainer: {
    flex: 1,
    paddingHorizontal: 20,
  },
  text: {
    color: 'black',
    fontSize: 20,
    fontWeight: 'bold',
  },
});
